<?php

namespace Drupal\entity_display_json\Controller;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\path_alias\AliasManagerInterface;

/**
 * Returns responses for Entity Display JSON routes.
 */
class EntityDisplayJsonController extends ControllerBase {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * The entity display repository.
   *
   * @var Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * JsonController constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *    The entity type manager.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *  The entity display repository.
   * @param Drupal\path_alias\AliasManagerInterface $aliasManager
   *  The entity type manager.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, EntityTypeManagerInterface $entity_type_manager, EntityDisplayRepositoryInterface $entity_display_repository, AliasManagerInterface $aliasManager) {
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityDisplayRepository = $entity_display_repository;
    $this->aliasManager = $aliasManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
      $container->get('entity_display.repository'),
      $container->get('path_alias.manager')
    );
  }

  /**
   * Get a paragraph and view mode and display it as JSON.
   * {@inheritdoc}
   */
  public function process_entity($entity, $display_id = 'default') {
    $basePath = \Drupal::request()->getSchemeAndHttpHost();
    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $data = [
      'entityType' => $entity_type,
      'bundle' => $bundle,
      'uuid' => $entity->uuid(),
      'id' => $entity->id(),
      'langcode' => $entity->language()->getId(),
    ];

    if ($entity_type == 'file') {
      $data['file_url'] = $entity->createFileUrl(FALSE);
      $data['file_name'] = $entity->getFilename();
      $data['file_size'] = $entity->getSize();
      $data['file_mime'] = $entity->getMimeType();
    }

    if ($entity->hasLinkTemplate('canonical')) {
      $path = $entity->toUrl()->getInternalPath();
      $data['path_alias'] = $basePath . $this->aliasManager->getAliasByPath('/' . $path);
    }
    // Since the entity in not fieldable, just return the data.
    if ($entity_type == 'view') {
      $data['display_id'] = $display_id;
      return $data;
    }

    $fields = $this->entityDisplayRepository->getViewDisplay($entity_type, $bundle, $display_id)->getComponents();
    foreach ($fields as $field_name => $field_value) {
      if (!$field_value['label']) {
        continue;
      }

      // Entity reference fields.
      if ($entity->$field_name->target_id) {
        $ref_entities = $entity->$field_name->referencedEntities();
        $display_id = $field_value["settings"]["view_mode"];
        foreach ($ref_entities as $key => $ref_entity) {
          if ($field_name == 'thumbnail') {
            $image_style = $field_value["settings"]["image_style"];
            $imageUri = $ref_entity->getFileUri();
            $data[$field_name][] = \Drupal\image\Entity\ImageStyle::load($image_style)->buildUrl($imageUri);
            continue;
          }
          $data[$field_name][$key] = self::process_entity($ref_entity, $display_id ?? $entity->$field_name->display_id);
          if (isset($ref_entity) && ($ref_entity->getEntityTypeId() == 'file') && ($ref_entity == $ref_entities[$key])) {
            $data[$field_name][$key]['uri'] =  $ref_entity->getFileUri();
          }
        }
      }

      // Normal value fields.
      if ($value = $entity->$field_name->value) {
        // Summary and or trimmed.
        $summary = $entity->$field_name->summary;
        if (isset($field_value['settings']['trim_length'])) {
          $length = $field_value['settings']['trim_length'];
          $format = $entity->$field_name->format;
          $value = strip_tags($value);
          $trimmed = text_summary($value, $format, $length);
        }
        if ($field_value['type'] === 'text_summary_or_trimmed') {
          $value = $summary ?? $trimmed ?? $value;
        }
        if ($field_value['type'] === 'text_trimmed') {
          $value = $trimmed ?? $value;
        }
        $data[$field_name] = $value;
      }

      // Special fields.
      $field_type = $entity->get($field_name)->getFieldDefinition()->getType();

      // Links
      if ($field_type === 'link_separate' || $field_type === 'link') {
        if (!$entity->$field_name->value && !$entity->$field_name->target_id) {
          $values = $entity->get($field_name)->getValue();
          foreach ($values as $value){
            $url = $value['uri'];
            if (strpos($url, 'internal:') === 0) {
              $url = $basePath . '/' . str_replace('internal:', '', $url);
            }
            if (strpos($url, 'entity:') === 0) {
              $url = $basePath . '/' . str_replace('entity:', '', $url);
            }
            $publicUrl = $url;
            $data[$field_name]['url'] = $publicUrl;
            $value['title'] ? $data[$field_name]['title'] = $value['title'] : NULL;
          }
        }
      }
    }
    return $data;
  }

  /**
   * Generates the JSON response based on entity display settings.
   *
   * @param string $entity_id
   *   The entity ID.
   * @param string $display_id
   *   The display ID.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   *   Thrown when the entity or display cannot be found.
   */
  public function build($entity_type, $uuid, $display_id = 'default') {
    $entity = reset($this->entityTypeManager->getStorage($entity_type)->loadByProperties(['uuid' => $uuid]));
    if (!$entity) {
      throw new NotFoundHttpException(sprintf('Entity with UUID %s not found.', $uuid));
    }
    $data = $this->process_entity($entity, $display_id);
    // Preprend apiVersion $data
    $data = ['apiVersion' => '1.0'] + $data;
    // Return the JSON response.
    return new JsonResponse($data);
  }
}
